class ExternalLinksPlugin {
	constructor(API, name, config) {
		this.API = API;
		this.name = name;
		this.config = config;
	}

	addModifiers() {
		if (this.config.scope === 'site' || this.config.scope === 'post') {
			this.API.addModifier('postText', this.modifyExternalLinks.bind(this), 1, this);
		}
		if (this.config.scope === 'site') {
			this.API.addModifier('htmlOutput', this.modifyExternalLinks.bind(this), 1, this);
		}
	}

	modifyExternalLinks(rendererInstance, text) {
		const baseUrl = rendererInstance.siteConfig.domain;
		const externalLinkRegex = new RegExp(`<a href="http(s)?:\/\/([^"]*)"(.*?)>(.*?)<\/a>`, 'g');
		const iconClass = this.config.iconChoice !== 'no-icon' ? this.config.iconChoice : '';
		const newClass = this.config.additionalClass ? ` ${this.config.additionalClass}` : '';
		const linkTitle = this.config.linkTitle;
		const overrideRel = this.config.overrideRel;
		const excludeSelectors = this.config.excludeSelectors ? this.config.excludeSelectors.split('\n').map(sel => sel.trim()).filter(Boolean) : [];

		return text.replace(externalLinkRegex, (match, protocol, link, attributes, innerHtml) => {
			try {
				const linkHostname = new URL(`http${protocol}://${link}`).hostname;
				const baseHostname = new URL(baseUrl).hostname;

				if (linkHostname === baseHostname) {
					return match;
				}
			} catch (error) {
				console.error('Invalid URL:', error);
				return match;
			}

			// Check for excluded selectors
			for (const selector of excludeSelectors) {
				const attributeRegex = new RegExp(`(class|id)="([^"]*\\b${selector}\\b[^"]*)"`);
				if (attributeRegex.test(attributes)) {
					return match;
				}
			}

			let relValues = attributes.match(/rel="([^"]*)"/)?.[1].split(' ') || [];
			if (overrideRel) {
				relValues = ['nofollow', 'noopener', 'noreferrer', 'sponsored', 'ugc'].filter(value => this.config[`add${value.charAt(0).toUpperCase() + value.slice(1)}`]);
			} else {
				['nofollow', 'noopener', 'noreferrer', 'sponsored', 'ugc'].forEach(value => {
					if (this.config[`add${value.charAt(0).toUpperCase() + value.slice(1)}`] && !relValues.includes(value)) {
						relValues.push(value);
					}
				});
			}

			let relAttribute = relValues.length > 0 ? `rel="${relValues.join(' ')}"` : '';
			let cleanAttributes = attributes.replace(/ rel="[^"]*"/, '');

			let titleAttribute = '';
			if (linkTitle) {
				titleAttribute = ` title="${linkTitle}"`;
				cleanAttributes = cleanAttributes.replace(/ title="[^"]*"/, '');
			}

			let classAttributeMatch = attributes.match(/class="([^"]*)"/);
			let existingClasses = classAttributeMatch ? classAttributeMatch[1].split(' ') : [];
			if (iconClass) {
				existingClasses.push('extlink', iconClass);
			} else {
				existingClasses.push('extlink');
			}
			existingClasses.push(newClass.trim());
			let classAttribute = `class="${existingClasses.join(' ')}"`;
			cleanAttributes = cleanAttributes.replace(/ class="[^"]*"/, '');

			if (!/<img|<video|<iframe|<svg|<object|<embed/i.test(innerHtml)) {
				return `<a href="http${protocol}://${link}" ${cleanAttributes.trim()} ${classAttribute} ${titleAttribute} ${relAttribute}>${innerHtml}</a>`;
			} else {
				return `<a href="http${protocol}://${link}" ${cleanAttributes.trim()} ${titleAttribute} ${relAttribute}>${innerHtml}</a>`;
			}
		});
	}

	addInsertions() {
		if (this.config.scope === 'site' || this.config.scope === 'post') {
			this.API.addInsertion('customHeadCode', this.addStyles.bind(this), 1, this);
		}
	}

	addStyles(rendererInstance) {
		if (this.config.scope === 'post' && rendererInstance.globalContext.context[0] !== 'post') {
			return '';
		}

		const iconMap = {
			'extlink-icon-1': 'url("data:image/svg+xml;utf8,<svg viewBox=\'0 0 24 24\' fill=\'none\' stroke=\'%23000000\' stroke-width=\'2\' stroke-linecap=\'round\' stroke-linejoin=\'round\' xmlns=\'http://www.w3.org/2000/svg\'><path d=\'M15 3h6v6\'/><path d=\'M10 14 21 3\'/><path d=\'M18 13v6a2 2 0 0 1-2 2H5a2 2 0 0 1-2-2V8a2 2 0 0 1 2-2h6\'/></svg>")',

			'extlink-icon-2': 'url("data:image/svg+xml;utf8,<svg viewBox=\'0 0 24 24\' fill=\'none\' stroke=\'%23000000\' stroke-width=\'2\' stroke-linecap=\'round\' stroke-linejoin=\'round\' xmlns=\'http://www.w3.org/2000/svg\'><path d=\'M21 13v6a2 2 0 0 1-2 2H5a2 2 0 0 1-2-2V5a2 2 0 0 1 2-2h6\'/><path d=\'m21 3-9 9\'/><path d=\'M15 3h6v6\'/></svg>")',

			'extlink-icon-3': 'url("data:image/svg+xml;utf8,<svg viewBox=\'0 0 24 24\' fill=\'none\' stroke=\'%23000000\' stroke-width=\'2\' stroke-linecap=\'round\' stroke-linejoin=\'round\' xmlns=\'http://www.w3.org/2000/svg\'><rect width=\'18\' height=\'18\' x=\'3\' y=\'3\' rx=\'2\'/><path d=\'M8 8h8v8\'/><path d=\'m8 16 8-8\'/></svg>")',

			'extlink-icon-4': 'url("data:image/svg+xml;utf8,<svg viewBox=\'0 0 24 24\' fill=\'none\' stroke=\'%23000000\' stroke-width=\'2\' stroke-linecap=\'round\' stroke-linejoin=\'round\' xmlns=\'http://www.w3.org/2000/svg\'><path d=\'M7 7h10v10\'/><path d=\'M7 17 17 7\'/></svg>")',

			'extlink-icon-5': 'url("data:image/svg+xml;utf8,<svg viewBox=\'0 0 24 24\' fill=\'none\' stroke=\'%23000000\' stroke-width=\'2\' stroke-linecap=\'round\' stroke-linejoin=\'round\' xmlns=\'http://www.w3.org/2000/svg\'><path d=\'M22 12A10 10 0 1 1 12 2\'/><path d=\'M22 2 12 12\'/><path d=\'M16 2h6v6\'/></svg>")',

			'extlink-icon-6': 'url("data:image/svg+xml;utf8,<svg viewBox=\'0 0 24 24\' fill=\'none\' stroke=\'%23000000\' stroke-width=\'2\' stroke-linecap=\'round\' stroke-linejoin=\'round\' xmlns=\'http://www.w3.org/2000/svg\'><path stroke=\'none\' d=\'M0 0h24v24H0z\' fill=\'none\'/><path d=\'M9 12h12\' /><path d=\'M17 16l4 -4l-4 -4\' /><path d=\'M12 3a9 9 0 1 0 0 18\' /></svg>")',

			'extlink-icon-7': 'url("data:image/svg+xml;utf8,<svg viewBox=\'0 0 24 24\' fill=\'none\' stroke=\'%23000000\' stroke-width=\'2\' stroke-linecap=\'round\' stroke-linejoin=\'round\' xmlns=\'http://www.w3.org/2000/svg\'><path stroke=\'none\' d=\'M0 0h24v24H0z\' fill=\'none\'/><path d=\'M3 12a9 9 0 1 0 18 0a9 9 0 0 0 -18 0\' /><path d=\'M15 9l-6 6\' /><path d=\'M15 15v-6h-6\' /></svg>")',

			'extlink-icon-8': 'url("data:image/svg+xml;utf8,<svg viewBox=\'0 0 24 24\' fill=\'none\' stroke=\'%23000000\' stroke-width=\'2\' stroke-linecap=\'round\' stroke-linejoin=\'round\' xmlns=\'http://www.w3.org/2000/svg\'><circle cx=\'11\' cy=\'11\' r=\'8\'/><path d=\'m21 21-4.3-4.3\'/></svg>")',

			'extlink-icon-9': 'url("data:image/svg+xml;utf8,<svg viewBox=\'0 0 24 24\' fill=\'none\' stroke=\'%23000000\' stroke-width=\'2\' stroke-linecap=\'round\' stroke-linejoin=\'round\' xmlns=\'http://www.w3.org/2000/svg\'><path stroke=\'none\' d=\'M0 0h24v24H0z\'/><path d=\'M13 4v4c-6.575 1.028 -9.02 6.788 -10 12c-.037 .206 5.384 -5.962 10 -6v4l8 -7l-8 -7z\'/></svg>")',

			'extlink-icon-10': 'url("data:image/svg+xml;utf8,<svg viewBox=\'0 0 24 24\' fill=\'none\' stroke=\'%23000000\' stroke-width=\'2\' stroke-linecap=\'round\' stroke-linejoin=\'round\' xmlns=\'http://www.w3.org/2000/svg\'><path stroke=\'none\' d=\'M0 0h24v24H0z\' fill=\'none\'/><path d=\'M9 15l6 -6\' /><path d=\'M11 6l.463 -.536a5 5 0 0 1 7.071 7.072l-.534 .464\' /><path d=\'M13 18l-.397 .534a5.068 5.068 0 0 1 -7.127 0a4.972 4.972 0 0 1 0 -7.071l.524 -.463\' /></svg>")',

			'extlink-icon-11': 'url("data:image/svg+xml;utf8,<svg viewBox=\'0 0 24 24\' fill=\'none\' stroke=\'%23000000\' stroke-width=\'2\' stroke-linecap=\'round\' stroke-linejoin=\'round\' xmlns=\'http://www.w3.org/2000/svg\'><path d=\'M10 13a5 5 0 0 0 7.54.54l3-3a5 5 0 0 0-7.07-7.07l-1.72 1.71\'/><path d=\'M14 11a5 5 0 0 0-7.54-.54l-3 3a5 5 0 0 0 7.07 7.07l1.71-1.71\'/></svg>")',

			'extlink-icon-12': 'url("data:image/svg+xml;utf8,<svg viewBox=\'0 0 24 24\' fill=\'none\' stroke=\'%23000000\' stroke-width=\'2\' stroke-linecap=\'round\' stroke-linejoin=\'round\' xmlns=\'http://www.w3.org/2000/svg\'> <path d=\'M12.298,7.202l-2.672-2.004c-.77-.577-1.842-.525-2.552.126l-4.128,3.784c-.413.379-.649.913-.649,1.473v4.621c0,2.4,2,4,4,4h8.571\' /><path d=\'M14.298,16.202h.571c2.286,0,2.286,3,0,3\' /><path d=\'M14.298,13.202h1.714c2.286,0,2.286,3,0,3h-1.714\' /><path d=\'M14.298,10.202h2.5c2.286,0,2.286,3,0,3h-2.5\' /><path d=\'M14.298,10.202h6.501c.828,0,1.499-.672,1.499-1.5h0c0-.828-.672-1.5-1.5-1.5h-11.5\' /></svg>")',
		};

		if (this.config.iconChoice === 'no-icon') {
			return ''; 
		}

		let position = this.config.iconPosition === 'after' ? 'after' : 'before';
		let marginValue = this.config.iconMargin || '5px';
		let iconVerticalPosition = this.config.iconVerticalPosition || '0';
		let iconSize = this.config.iconSize || '16px';
		let iconColor = this.config.iconColorSelect === 'currentColor' ? 'currentColor' : this.config.iconColor || 'currentColor';

		let css = `
			.extlink::${position} {
				background-color: ${iconColor};
				content: "";
				display: inline-block;
				height: ${iconSize}; 
				margin-${position === 'after' ? 'left' : 'right'}: ${marginValue};
				position: relative;
				top: ${iconVerticalPosition}px;
				width: ${iconSize}; 
			}
		`;

		Object.keys(iconMap).forEach(iconKey => {
			if (this.config.iconChoice === iconKey) {
				css += `.extlink.${iconKey}::${position} { mask-image: ${iconMap[iconKey]}; }`;
			}
		});

		return `<style>${css}</style>`;
	}
}

module.exports = ExternalLinksPlugin;
